import base64
import logging
import os
from typing import Dict, List, Union

import requests
import vercel_blob
from mcp.server import FastMCP

logger = logging.getLogger(__name__)
mcp = FastMCP(
    "vercel_blob_server",
    capabilities={
        "resources": {}  # 启用resources功能
    }
)


@mcp.tool()
def put_blob(filename: str, content: str, add_random_suffix: bool = False, cache_max_age: str = "31536000") -> Dict[
    str, str]:
    """
    Write content to Vercel Blob storage.
    
    Args:
        filename (str): The path/name for the blob file (e.g., 'folder/file.txt')
        content (str): The content to be written to the blob
        add_random_suffix (bool): Whether to add a random suffix to the filename
        cache_max_age (str): Cache control max age value in seconds
        
    Returns:
        Dict[str, str]: A dictionary containing either success message with URL or error message
        
    Example:
        >>> result = put_blob(
        ...     "docs/readme.md",
        ...     "# Hello World",
        ...     add_random_suffix=True
        ... )
        >>> print(result)
        {
            "success": "Content written to blob successfully",
            "url": "https://example.blob.vercel-storage.com/docs/readme-123456.md"
        }
    """
    try:
        result = vercel_blob.put(
            filename,
            content.encode("utf-8"),
            options={
                'addRandomSuffix': str(add_random_suffix).lower(),
                'cacheControlMaxAge': cache_max_age
            }
        )

        return {
            "success": "Content written to blob successfully",
            "url": result['url']
        }

    except Exception as e:
        error_msg = f"Failed to write to blob: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def list_blobs(prefix: str = "", limit: int = 1000, cursor: str = None, mode: str = "expanded") -> Dict[str, any]:
    """
    List blobs in Vercel Blob storage with optional filtering and pagination.
    
    Args:
        prefix (str): Filter blobs by prefix path (e.g., 'images/', 'docs/')
        limit (int): Maximum number of items to return (default: 1000)
        cursor (str): Pagination cursor from previous response
        mode (str): Response format - 'expanded' or 'folded' (default: 'expanded')
                   In folded mode, blobs inside folders are folded into single folder entries
        
    Returns:
        Dict: A dictionary containing the list of blobs or error message
        
    Example:
        >>> result = list_blobs(prefix="docs/", limit=10)
        >>> print(result)
        {
            "blobs": [
                {
                    "url": "https://example.blob.vercel-storage.com/docs/file1.txt",
                    "pathname": "docs/file1.txt",
                    "size": 1234,
                    "uploadedAt": "2024-03-14T12:00:00Z"
                },
                ...
            ],
            "hasMore": true,
            "cursor": "next-page-token"
        }
    """
    try:
        options = {
            'prefix': prefix,
            'limit': str(limit),
            'mode': mode
        }

        if cursor:
            options['cursor'] = cursor

        result = vercel_blob.list(options)

        # Transform the response into a more usable format
        blob_list = [{
            'url': blob['url'],
            'pathname': blob['pathname'],
            'size': blob['size'],
            'uploadedAt': blob['uploadedAt']
        } for blob in result['blobs']]

        response = {
            "blobs": blob_list,
            "hasMore": bool(result.get('cursor')),
        }

        if result.get('cursor'):
            response["cursor"] = result['cursor']

        return response

    except Exception as e:
        error_msg = f"Failed to list blobs: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def head_blob(url: str) -> Dict[str, any]:
    """
    Get metadata of a blob object at the specified URL.
    
    Args:
        url (str): The URL of the blob to get metadata for
        
    Returns:
        Dict: A dictionary containing either the metadata or an error message
        
    Example:
        >>> result = head_blob("https://example.blob.vercel-storage.com/docs/file.txt")
        >>> print(result)
        {
            "metadata": {
                "size": 1234,
                "uploadedAt": "2024-03-14T12:00:00Z",
                ...
            }
        }
    """
    try:
        result = vercel_blob.head(url)
        logger.info(f"Successfully retrieved metadata for blob: {url}")
        return {"metadata": result}
    except Exception as e:
        error_msg = f"Failed to get blob metadata: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def delete_blobs(urls: Union[str, List[str]]) -> Dict[str, any]:
    """
    Delete one or more blobs from storage.
    
    Args:
        urls (str or List[str]): Single URL or list of URLs to delete
        
    Returns:
        Dict: A dictionary containing either success message or error
        
    Example:
        >>> result = delete_blobs("https://example.blob.vercel-storage.com/docs/file.txt")
        >>> print(result)
        {
            "success": "Successfully deleted 1 blob(s)"
        }
    """
    try:
        result = vercel_blob.delete(urls)
        num_deleted = len(urls) if isinstance(urls, list) else 1
        logger.info(f"Successfully deleted {num_deleted} blob(s)")
        return {"success": f"Successfully deleted {num_deleted} blob(s)"}
    except Exception as e:
        error_msg = f"Failed to delete blob(s): {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def copy_blob(blob_url: str, to_path: str, add_random_suffix: bool = False) -> Dict[str, any]:
    """
    Copy a blob from source URL to a new path.
    
    Args:
        blob_url (str): Source blob URL
        to_path (str): Destination path for the copy
        add_random_suffix (bool): Whether to add random suffix to destination path
        
    Returns:
        Dict: A dictionary containing either the new blob URL or error message
        
    Example:
        >>> result = copy_blob(
        ...     "https://example.blob.vercel-storage.com/docs/old.txt",
        ...     "docs/new.txt"
        ... )
        >>> print(result)
        {
            "success": "Blob copied successfully",
            "url": "https://example.blob.vercel-storage.com/docs/new.txt"
        }
    """
    try:
        result = vercel_blob.copy(
            blob_url,
            to_path,
            options={'addRandomSuffix': str(add_random_suffix).lower()}
        )
        logger.info(f"Successfully copied blob from {blob_url} to {to_path}")
        return {
            "success": "Blob copied successfully",
            "url": result['url']
        }
    except Exception as e:
        error_msg = f"Failed to copy blob: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def download_blob(url: str, save_path: str = '') -> Dict[str, str]:
    """
    Download a blob from storage to local filesystem.
    
    Args:
        url (str): URL of the blob to download
        save_path (str): Local directory path to save the file (must end with '/')
        
    Returns:
        Dict: A dictionary containing either success message or error
        
    Example:
        >>> result = download_blob(
        ...     "https://example.blob.vercel-storage.com/docs/file.txt",
        ...     "downloads/"
        ... )
        >>> print(result)
        {
            "success": "File downloaded successfully to downloads/file.txt"
        }
    """
    try:
        vercel_blob.download_file(url, save_path)
        filename = url.split('/')[-1]
        full_path = os.path.join(save_path, filename)
        logger.info(f"Successfully downloaded blob to {full_path}")
        return {"success": f"File downloaded successfully to {full_path}"}
    except Exception as e:
        error_msg = f"Failed to download blob: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.tool()
def read_blob(url: str) -> Dict[str, any]:
    """
    Read content directly from a blob URL.
    
    Args:
        url (str): The URL of the blob to read
        
    Returns:
        Dict: A dictionary containing either the content or error message
        
    Example:
        >>> result = read_blob("https://example.blob.vercel-storage.com/docs/file.txt")
        >>> print(result)
        {
            "content": "Hello World!",
            "content_type": "text/plain"
        }
    """
    try:
        response = requests.get(f"{url}?download=1")
        response.raise_for_status()

        content_type = response.headers.get('content-type', 'application/octet-stream')

        # 如果是文本类型，直接返回解码后的内容
        if 'text' in content_type or content_type in ['application/json', 'application/xml']:
            content = response.text
        else:
            # 对于二进制内容，返回base64编码
            content = base64.b64encode(response.content).decode('utf-8')

        logger.info(f"Successfully read blob content from: {url}")
        return {
            "content": content,
            "content_type": content_type
        }

    except Exception as e:
        error_msg = f"Failed to read blob content: {str(e)}"
        logger.error(error_msg)
        return {"error": error_msg}


@mcp.resource("vercel-blob://{uri}")
def read_blob_file(uri: str) -> Dict[str, any]:
    """
    Read a resource from Vercel Blob storage.
    
    Args:
        uri (str): The URI of the resource to read (format: vercel-blob://path/to/file)
        
    Returns:
        Dict[str, any]: The resource content and metadata
    """
    try:
        blob_path = uri[len('vercel-blob://'):]

        # 获取blob列表以找到对应的URL
        result = vercel_blob.list({'prefix': blob_path})
        matching_blobs = [b for b in result['blobs'] if b['pathname'] == blob_path]

        if not matching_blobs:
            raise ValueError(f"Resource not found: {uri}")

        blob = matching_blobs[0]

        # 读取内容
        response = requests.get(f"{blob['url']}?download=1")
        response.raise_for_status()

        content_type = response.headers.get('content-type', 'application/octet-stream')

        # 根据内容类型决定如何处理内容
        if 'text' in content_type or content_type in ['application/json', 'application/xml']:
            content = response.text
            is_binary = False
        else:
            content = base64.b64encode(response.content).decode('utf-8')
            is_binary = True

        logger.info(f"Successfully read resource: {uri}")

        return {
            'uri': uri,
            'mimeType': content_type,
            'content': content,
            'binary': is_binary
        }

    except Exception as e:
        error_msg = f"Failed to read resource: {str(e)}"
        logger.error(error_msg)
        raise Exception(error_msg)


def list_resources() -> List[Dict[str, str]]:
    """
    List available resources from Vercel Blob storage.

    Returns:
        List[Dict[str, str]]: A list of resources with their URIs and metadata
    """
    try:
        # 获取所有blob列表
        result = vercel_blob.list({
            'limit': 1000,
            'mode': 'expanded'
        })

        # 转换为MCP资源格式
        resources = []
        for blob in result['blobs']:
            resources.append({
                'uri': f"vercel-blob://{blob['pathname']}",  # 使用自定义URI scheme
                'name': blob['pathname'].split('/')[-1],  # 使用文件名作为资源名
                'description': f"Size: {blob['size']} bytes, Uploaded: {blob['uploadedAt']}",
                'mimeType': 'application/octet-stream'  # 默认MIME类型
            })

        logger.info(f"Successfully listed {len(resources)} resources")
        return resources

    except Exception as e:
        error_msg = f"Failed to list resources: {str(e)}"
        logger.error(error_msg)
        raise Exception(error_msg)


def run_server():
    """
    Start the MCP server after checking for required environment variables.
    Raises SystemExit if required environment variables are missing.
    """
    if not os.getenv('BLOB_READ_WRITE_TOKEN'):
        error_msg = "Error: BLOB_READ_WRITE_TOKEN environment variable is not set"
        logger.error(error_msg)
        raise SystemExit(error_msg)

    mcp.run(transport="stdio")


if __name__ == "__main__":
    run_server()
